<?php

/**************************************************************************
 *
 * file.php
 *
 * This is a plugin for phpWebFileManager that allows you to restrict
 * access to View files.  
 *
 * Although it wouldn't technically be required, it works best to set up a
 * mod_rewrite rule in apache so that when a user choses to download a file
 * their browser doesn't ask them to save "file.php", but the actual file
 * name.  You can set up mod_rewrite like this:
 *
 * RewriteEngine On RewriteRule ^/fm/file(.*)$ /fm/file.php?path=$1 [L]
 *
 * where /fm/ is where phpWebFileManager is installed.
 *
 * Then change $fm_cfg['url'] in your config.inc.php file * to be "file/".
 * If you can't use mod_rewrite for whatever reason, set $fm_cfg['url'] to
 * "file.php?path=". (untested) 
 *
 * You should also adjust the $APACHE_MIME_TYPES variable below to  point
 * at your apache "mime.types" file. 
 *
 * Tony J. White, tjw@tjw.org, 2002-04-15
 **************************************************************************/

/*
 * phpWebFileManager initialization
 *
 * Config file is read from init.inc.php file.
 */

$phpWebFileManager_INIT_FILE = dirname(__FILE__)
	. (strlen(dirname(__FILE__)) > 0 ? '/' : '')
	. 'init.inc.php';

	if (! @file_exists($phpWebFileManager_INIT_FILE)) {
		exit;
	}

require_once $phpWebFileManager_INIT_FILE;
require_once $PN_PathPrefix . 'lib/fnc.append_slash.php';

// note that this constant will not remain in future
// it will be moved into config file, where array of possible
// Apache mime.types locations will be present
$APACHE_MIME_TYPES = '/etc/httpd/conf/apache-mime.types';

/*
 * Permission check
 */
if (! $fm_cfg['perm']['file']['view']) {
	echo 'Insufficient privileges for file viewing.';
} else {

	/*
	 * Parsing of apache-mime.types file
	 */

	if (is_readable($GLOBALS["APACHE_MIME_TYPES"])) {
		$mime_types = file($GLOBALS["APACHE_MIME_TYPES"]);
		if (is_array($mime_types)) {

			while (list(,$line) = each($mime_types)) {
				if ($line[0] == '#')
					continue;

				$parts = preg_split("/(\s+)/", $line);
				if (is_array($parts) && sizeof($parts) > 2) {
					list(,$mime) = each($parts);
					while (list(,$ext) = each($parts)) {
						if (trim($ext) != '')
							$mimes_by_ext[$ext] = $mime;
					}
				}
			}

		}
	}

	/*
	 * Creating $file variable
	 */

	$file = append_slash($fm_cfg['dir']['root']) . substr($path, 1);

	/*
	 * Writting file on output
	 */

	if (file_exists($file) && is_readable($file)) {
		header('Content-length: '.filesize($file));
		if (preg_match("/.*\.([a-z]{2,5})$/i", $file, $matches)
				&& trim($mimes_by_ext[$matches[1]]) != '') {
			header('Content-type: '.trim($mimes_by_ext[$matches[1]]));
		} else {
			header('Content-type: text/plain');
		}

		readfile($file);
	} else {
		echo "File '$file' not found or is not readable.";
	}

}

?>
