<?php

/*
 * phpWebFileManager - simple file management PHP script
 *
 * phpWebFileManager.php - main script file
 * ____________________________________________________________
 *
 * Developed by Ondrej Jombik <nepto@pobox.sk>
 * Copyright (c) 2001-2002 Platon SDG, http://www.platon.sk/
 * All rights reserved.
 *
 * See README file for more information about this software.
 * See COPYING file for license information.
 *
 * Download the latest version from
 * http://www.platon.sk/projects/phpWebFileManager/
 */

/* $Id: index.php,v 1.26 2002/07/21 22:49:42 nepto Exp $ */

/*
 * phpWebFileManager initialization
 *
 * Config file is read from init.inc.php file.
 */

$phpWebFileManager_INIT_FILE = dirname(__FILE__)
	. (strlen(dirname(__FILE__)) > 0 ? '/' : '')
	. 'init.inc.php';

	if (! @file_exists($phpWebFileManager_INIT_FILE)) {
		exit;
	}

require_once $phpWebFileManager_INIT_FILE;

/*
 * Libraries function inclusion
 */

require_once $PN_PathPrefix . 'lib/fnc.append_slash.php';
require_once $PN_PathPrefix . 'lib/fnc.get_path_without_scriptname.php';
require_once $PN_PathPrefix . 'lib/fnc.glue_url.php';
require_once $PN_PathPrefix . 'lib/fnc.write_origvars_html.php';

/*
 * echo_message()
 *
 * Writes message on output.
 */

function echo_message($message)
{
	echo "<small>$message</small>";
}


/*
 * Main job
 */

/* Hidden parameters */
$fm_origvars           = append_slash($fm_cfg['origvars'], '&');
$fm_cfg['url']         = append_slash($fm_cfg['url']);
$fm_cfg['dir']['root'] = append_slash($fm_cfg['dir']['root']);
if (strlen($fm_cfg['format']['date']) <= 0) {
	$fm_cfg['format']['date'] = _FM_DATE_FORMAT;
}

/*
 * Striping slashes.
 */

$fm_action   = isset($fm_action) ? stripslashes($fm_action) : '';
$submit      = isset($submit) ? stripslashes($submit) : '';
$fm_dir      = isset($fm_dir) ? stripslashes($fm_dir) : '';
$fm_filename = isset($fm_filename) ? stripslashes($fm_filename) : '';
$fm_new_filename = isset($fm_new_filename)
	? stripslashes($fm_new_filename) : '';

/*
 * Symlink detection (must be before following directory modifying).
 */

if (! $fm_cfg['perm']['dir']['follow_symlinks']) {
	$fm_test_symlink = $fm_dir;

	while ($fm_test_symlink != '') {
		if ((@file_exists($fm_test_symlink) && @is_link($fm_test_symlink))
			|| (@file_exists($fm_test_symlink) && @is_link("$fm_test_symlink/"))) {
			$fm_dir = $fm_cfg['dir']['start'];
			break;
		}

		$fm_test_symlink = substr($fm_test_symlink,
				0, intval(strrpos($fm_test_symlink, '/')));
	}
}

if (! isset($fm_dir) || ! $fm_cfg['perm']['dir']['enter']) {
	$fm_dir = $fm_cfg['dir']['start'];
}

/* 
 * Directory modifying. Dirty, but safe.
 */

$fm_dir = preg_replace('/\/\/+/','/', "$fm_dir/");
$fm_dir = preg_replace("/^(|.*\/)\.\.\/([^\.\/][^\.\/]|[^\/][^\.\/]|[^\/][^\.\/]|[^\/]{3,})\/(.*)$/", "\\1\\3", $fm_dir);
$fm_dir = preg_replace("/^(|.*\/)([^\/\.][^\.\/]|[^\/][^\.\/]|[^\/][^\.\/]|[^\/]{3,})\/\.\.\/(.*)$/", "\\1\\3", $fm_dir);

while (preg_match("/^(|.*\/)\.\.\/(.*)$/", $fm_dir)) {
	$fm_dir = preg_replace("/^(|.*\/)\.\.\/(.*)$/", "\\1\\2", $fm_dir);
}
while (strlen($fm_dir) > 0 && $fm_dir[0] == '/') {
	$fm_dir = substr($fm_dir, 1, strlen($fm_dir));
}

/*
 * Change to root directory.
 */

/* Store the starting working directory. */
$fm_orig_dir = getcwd();

if (@is_dir($fm_cfg['dir']['root'])) {
	chdir($fm_cfg['dir']['root']);
}

$fm_root_dir = getcwd();

if (@file_exists($fm_dir) && @is_dir($fm_dir)) {
	chdir($fm_dir);

	/* On some *BSD systems with PHP version <= 4.0.6
	   function chdir() is removing last '/' character
	   in $fm_dir variable, so we must improve it. */
	$fm_dir = append_slash($fm_dir);
}
else  {
	$fm_dir = '';
}

/*
 * Prepare URL & HTML encode of some variables.
 */

$fm_filename_he = htmlentities($fm_filename);
$fm_filename_he2 = str_replace(' ', '&nbsp;', $fm_filename_he);
//$fm_filename_ue = htmlentities(rawurlencode($fm_filename));
$fm_dir_he = htmlentities($fm_dir);
$fm_dir_he2 = str_replace(' ', '&nbsp;', $fm_dir_he);
$fm_dir_ue = htmlentities(rawurlencode($fm_dir));
$PHP_SELF_origvars_ue = htmlentities("$PHP_SELF?$fm_origvars");

/*
 * Title writting with current working direcotry information.
 */

if (defined('LOADED_AS_MODULE')) {
	OpenTable();
	echo '<font class="pn-title">'._FM_FILE_MANAGER
		. (strlen($fm_dir) > 0 ? ": &nbsp;$fm_dir_he" : '') 
		. '</font>';
	CloseTable();
	OpenTable();
}
else {
	if (strlen($fm_dir) > 0)
		echo "<h3>$fm_dir_he2</h3>";
}

/*
 * Form
 */

echo '<form enctype="multipart/form-data" method=post'
. ' action="' . htmlentities($PHP_SELF) . '">';

write_origvars_html($fm_origvars);

echo '<input type=hidden value="'.$fm_filename_he.'" name="fm_filename">';
echo '<input type=hidden value="'.$fm_dir_he.'" name="fm_dir">';

/* PN: PostNuke module calls */
if (defined('LOADED_AS_MODULE')) {
	echo '<input type=hidden name="op" value="modload">';
	echo '<input type=hidden name="name" value="'.$ModName.'">';
	echo '<input type=hidden name="file" value="index">';
}

/*
 * confirm delete files action
 */
if ($fm_action == "confirm_delete_file") {
	echo "<input type=hidden value=delete_file name=fm_action>";

	echo _FM_REALLY_DELETE
		. "<br><br><i>$fm_filename_he2</i><br><br>";
	echo $fm_dir == ''
		? _FM_FROM_ROOT_DIR . "?<br><br>"
		: _FM_FROM_DIR
		. "<br><br><i>$fm_dir_he2</i><br><br>";
	echo "<input type=submit name=submit value=\""
		. _FM_FILE_DELETE . "\">&nbsp;"
		. "<input type=submit name=submit value=\""
		. _FM_CANCEL . "\">";
}

/*
 * confirm remove directory action
 */
elseif ($fm_action == "confirm_remove_directory") {
	echo "<input type=hidden value=remove_directory name=fm_action>";

	echo _FM_REALLY_REMOVE
		. "<br><br><i>$fm_filename_he2</i><br><br>";
	echo _FM_MUST_BE_EMPTY . "<br><br>";
	echo "<input type=submit name=submit value=\""
		. _FM_DIR_REMOVE . "\">&nbsp;"
		. "<input type=submit name=submit value=\""
		. _FM_CANCEL . "\">";
}

/*
 * confirm rename files action and confirm rename directory action
 */
elseif ($fm_action == "confirm_rename_file"
		|| $fm_action == "confirm_rename_directory") {
	echo "<input type=hidden value=\"";
	echo $fm_action == "confirm_rename_directory" 
		? "rename_directory" : "rename_file";
	echo "\" name=fm_action>";

	echo _FM_RENAME_FROM
		. "<br><br><i>$fm_filename_he2</i><br><br>"
		. _FM_RENAME_TO
		. "<br><br>";
	echo "<input type=text value=\"\" name=fm_new_filename><br><br>";
	echo "<input type=submit name=submit value=\""
		. ($fm_action == "confirm_rename_directory"
				? _FM_DIR_RENAME : _FM_FILE_RENAME)
		. "\">&nbsp;"
		. "<input type=submit name=submit value=\""
		. _FM_CANCEL . "\">";
}

/*
 * confirm create directory and confirm create file actions
 */
elseif ($fm_action == "confirm_create_directory" 
		|| $fm_action == "confirm_create_file") {
	echo "<input type=hidden value=";
	echo $fm_action == "confirm_create_directory" 
		? "create_directory" : "create_file";
	echo " name=fm_action>";

	echo _FM_ENTER_NAME . ":<br><br>";
	echo "<input type=text value=\"\" name=fm_new_filename><br><br>";
	echo "<input type=submit name=submit value=\""
		. ($fm_action == "confirm_create_directory"
				? _FM_DIR_CREATE : _FM_FILE_CREATE)
		. "\">&nbsp;"
		. "<input type=submit name=submit value=\""
		. _FM_CANCEL . "\">";
}

/*
 * confirm upload file action
 */
elseif ($fm_action == "confirm_upload_file") {
	echo "<input type=hidden value=\"upload_file\" name=fm_action>";

	echo _FM_SELECT_LOCAL . ":<br><br>";
	echo "<input type=file value=\"\" name=fm_userfile><br><br>";
	echo "<input type=submit name=submit value=\""
		. _FM_FILE_UPLOAD . "\">&nbsp;"
		. "<input type=submit name=submit value=\""
		. _FM_CANCEL . "\">";
}

else {

	/*
	 * delete action
	 */
	if ($fm_action == 'delete_file' && $fm_cfg['perm']['file']['delete']) {
		if ($submit == _FM_FILE_DELETE) {
			if (@unlink($fm_filename) == false)
				echo_message(_FM_FILE_DELETE_ERR);
			else
				echo_message(_FM_FILE_DELETE_OK);
		}
		else echo_message(_FM_CANCELED);
		echo '<br><br>';
	}

	/*
	 * remove direcotry
	 */
	if ($fm_action == 'remove_directory' && $fm_cfg['perm']['dir']['remove']) {
		if ($submit == _FM_DIR_REMOVE) {
			if (@rmdir($fm_filename) == 0)
				echo_message(_FM_DIR_REMOVE_ERR);
			else
				echo_message(_FM_DIR_REMOVE_OK);
		}
		else echo_message(_FM_CANCELED);
		echo '<br><br>';
	}

	/*
	 * rename action
	 */
	if (($fm_action == 'rename_directory' && $fm_cfg['perm']['dir']['rename'])
			|| ($fm_action == 'rename_file' && $fm_cfg['perm']['file']['rename'])) {
		if ($fm_new_filename != '' &&
				(($submit == _FM_DIR_RENAME && $fm_action == "rename_directory")
				 || ($submit == _FM_FILE_RENAME && $fm_action == 'rename_file')
				)) {
			if (@rename($fm_filename, $fm_new_filename) == false)
				echo_message(_FM_RENAME_ERR);
			else
				echo_message(_FM_RENAME_OK);
		}
		else echo_message(_FM_CANCELED);
		echo '<br><br>';
	}

	/*
	 * create directory and create file actions
	 */
	if (($fm_action == 'create_directory' && $fm_cfg['perm']['dir']['create']) 
			|| ($fm_action == 'create_file' && $fm_cfg['perm']['file']['create'])) {
		if ($fm_new_filename != '' &&
				(($submit == _FM_DIR_CREATE && $fm_action == 'create_directory')
				 || ($submit == _FM_FILE_CREATE && $fm_action == 'create_file'))
		   ) {
			if ($fm_action == 'create_directory') {
				if (@mkdir($fm_new_filename, $fm_cfg['mode']['dir']) == false)
					echo_message(_FM_DIR_CREATE_ERR);
				else
					echo_message(_FM_DIR_CREATE_OK);
			}
			else {
				if (file_exists($fm_new_filename)) 
					echo_message(_FM_FILE_CREATE_ERR1);
				else {
					if (($fh = @fopen($fm_new_filename, "w")) == false) 
						echo_message($phpWebFileManager_LANG[file_create_err2]);
					else {
						fclose($fh);
						echo_message(_FM_FILE_CREATE_OK);

						if ($fm_cfg['mode']['file'] != 0) {
							if (@chmod($fm_new_filename, $fm_cfg['mode']['file']) == false) 
								echo_message(' '.  _chmod_err);
							else
								echo_message(' '.  _chmod_ok);
						}
					}
				}
			}
		}
		else echo_message(_FM_CANCELED);
		echo '<br><br>';
	}

	/*
	 * file upload action
	 */
	if ($fm_action == 'upload_file' && $fm_cfg['perm']['file']['upload']) {
		if ($submit == _FM_FILE_UPLOAD && $fm_userfile_name != '') {

			$tmp_target_filename = $target_filename = $fm_userfile_name;

			$i = 2;
			while (file_exists($tmp_target_filename)) {
				if ($i == 2) {
					echo_message(_FM_FILENAME_CHANGED);
					echo '<br>';
				}
				$tmp_target_filename = $target_filename . '-' . $i;
				$i++;
			}

			$target_filename = $tmp_target_filename;

			//echo("fm_userfile = $fm_userfile<br>");
			//echo("fm_userfile_name = $fm_userfile_name<br>");
			//echo("target_filename = $target_filename<br>");

			if (@copy($fm_userfile, $target_filename)) {

				echo_message(_FM_FILE_UPLOAD_OK);

				if ($fm_cfg['mode']['file'] != 0) {
					if (@chmod($target_filename, $fm_cfg['mode']['file']) == false) 
						echo_message(' '._FM_CHMOD_ERR);
					else
						echo_message(' '._FM_CHMOD_OK);
				}
			}
			else 
				echo_message(_FM_FILE_UPLOAD_ERR);
		}
		else echo_message(_FM_CANCELED);

		echo '<br><br>';
	}


	echo '<table cellpadding="2" class=phpWebFileManager>';

	$line = 0;

	if (strlen(getcwd()) > strlen($fm_root_dir)) {
		echo '<tr valign=middle';
		if (! empty($fm_cfg['color']['odd']))
			echo ' bgcolor="'.$fm_cfg['color']['odd'].'"';
		echo '>';

		echo "<td align=center>"
			. "<a href=\"$PHP_SELF_origvars_ue" . "fm_dir=$fm_dir_ue..\">"
			. "<img alt=\"" . _FM_BACK
			. "\" border=0 src=\""
			. (defined('LOADED_AS_MODULE') ? "modules/$ModName/" : '')
			. "icons/back.gif\"></a>" . "</td>";

		echo "<td width=\"100%\" colspan=\"6\">"
			. "<a href=\"$PHP_SELF_origvars_ue" . "fm_dir=$fm_dir_ue..\">"
			. "(" . _FM_PARENT_DIR . ")</a>"
			. "</td>";

		echo "</tr>\n";

		$line++;
	}

	/*
	 * Column display counting.
	 */

	$rename_column = $fm_cfg['perm']['dir']['rename']
		|| $fm_cfg['perm']['file']['rename'];
	$delete_column = $fm_cfg['perm']['dir']['remove']
		|| $fm_cfg['perm']['file']['delete'];
	$view_column = $fm_cfg['perm']['dir']['enter']
		|| $fm_cfg['perm']['file']['view'];
	$last_row = $fm_cfg['perm']['dir']['create'] 
		|| $fm_cfg['perm']['file']['create']
		|| $fm_cfg['perm']['file']['upload'];

	/*
	 * Two steps for() loop.
	 * One for directories and one for files.
	 */
	for ($k = 0; $k < 2; $k++) {

		$d = dir('./');

		/* Order in alphabetical order. */
		$listing = array();
		while ($entry = $d->read()) 
			array_push($listing, $entry);
		sort($listing);

		while ($entry = array_shift($listing)) {
			$entry_he  = htmlentities($entry);
			$entry_he2 = str_replace(' ', '&nbsp;', $entry_he);
			$entry_ue  = htmlentities(rawurlencode($entry));

			// Dirty fastfix by Asbjrn Vrnes <avaernes@online.no>
			$entry_ue = str_replace('&ouml;', '%F6', $entry_ue);
			$entry_ue = str_replace('&aelig;', '%E6', $entry_ue);
			$entry_ue = str_replace('&oslash;', '%F8', $entry_ue);
			$entry_ue = str_replace('&aring;', '%E5', $entry_ue);
			$entry_ue = str_replace('&Aelig;', '%C6', $entry_ue);
			$entry_ue = str_replace('&Oslash;', '%D8', $entry_ue);
			$entry_ue = str_replace('&Aring;', '%C5', $entry_ue);

			if ($entry != '.' && $entry != '..' && 
					(($k == 0 && is_dir($entry))
					 || ($k == 1 && ! is_dir($entry)))) {

				echo '<tr valign=middle';
				$line++;
				if ($line % 2) {
					if (! empty($fm_cfg['color']['odd']))
						echo ' bgcolor="'.$fm_cfg['color']['odd'].'"';
				} else {
					if (! empty($fm_cfg['color']['even']))
						echo ' bgcolor="'.$fm_cfg['color']['even'].'"';
				}
				echo '>';

				if (is_dir($entry)) {
					if ($fm_cfg['show']['icons']) {
						echo "<td align=center>";
						if ($fm_cfg['perm']['dir']['enter'] && ! is_link($entry)) 
							echo "<a href=\"$PHP_SELF_origvars_ue" 
								. "fm_dir=$fm_dir_ue$entry_ue\">";

						echo (is_link($entry)
								?  '<b>~</b>'
								: "<img alt=\"" . _FM_DIR
								. "\" border=0 src=\""
								. (defined('LOADED_AS_MODULE')
									? "modules/$ModName/" : '')
								. "icons/dir.gif\">");

						if ($fm_cfg['perm']['dir']['enter'] && ! is_link($entry)) 
							echo '</a>';

						echo '</td>';
					}

					echo "<td width=\"100%\">";

					if ($fm_cfg['perm']['dir']['enter']
							&& ($fm_cfg['perm']['dir']['follow_symlinks']
								|| ! is_link($entry))) {
						echo "<a href=\"$PHP_SELF_origvars_ue" 
							. "fm_dir=$fm_dir_ue$entry_ue\">"
							. $entry_he2 .  "</a>";
					}
					else {
						echo $entry_he2;
					}
				}
				else {
					if ($fm_cfg['show']['icons']) {
						echo '<td align=center>';
						echo (is_link($entry) ? '<b>@</b>' : '&nbsp;');
						echo '</td>';
					}
					echo '<td width="100%">' . $entry_he2;
				}

				echo '</td>';

				if (! is_dir($entry)) {
					if ($fm_cfg['show']['size']) {
						echo '<td align=right nowrap><small>';
						$size = filesize($entry);
						$unit = "B";
						if ( $size > 1073741824 ) {
							$size = round ($size / 1073741824, 1);
							$unit = "GB";
						} elseif ( $size > 1048576 ) {
							$size = round ($size / 1048576, 1);
							$unit = "MB";
						} elseif ( $size > 1024 ) {
							$size = round ($size / 1024, 1);
							$unit = "kB";
						}
						echo $size . '&nbsp;' . $unit;
						echo '</small></td>';
					}

					if ($fm_cfg['show']['date']) {
						echo '<td align=center nowrap><small>';
						echo date($fm_cfg['format']['date'], filemtime($entry));
						echo '</small></td>';
					}

					if ($rename_column) {
						echo "<td><small>";
						if ($fm_cfg['perm']['file']['rename'])
							echo "[<a href=\"$PHP_SELF_origvars_ue"
								. "fm_dir=$fm_dir_ue&"
								. "fm_action=confirm_rename_file"
								. "&fm_filename=$entry_ue\">"
								. _FM_FILE_RENAME
								. "</a>]";
						echo '</small></td>';
					}

					if ($delete_column) {
						echo '<td><small>';
						if ($fm_cfg['perm']['file']['delete']) 
							echo "[<a href=\"$PHP_SELF_origvars_ue"
								. "fm_dir=$fm_dir_ue"
								. "&fm_action=confirm_delete_file"
								. "&fm_filename=$entry_ue\">"
								. _FM_FILE_DELETE
								. "</a>]";
						echo '</small></td>';
					}

					$url = parse_url("http://${HTTP_HOST}${REQUEST_URI}");
					$url['path'] = get_path_without_scriptname($url) 
						. (strlen($fm_cfg['url']) > 0 
								? $fm_cfg['url'] : $fm_cfg['dir']['root'])
						. $fm_dir_ue . $entry_ue;
					unset($url['query']);
					unset($url['fragment']);

					if ($view_column) {
						echo '<td><small>';
						if ($fm_cfg['perm']['file']['view'])
							echo "[<a target=_blank href=\""
								. str_replace('%2F', '/', glue_url($url)) . "\">"
								. _FM_FILE_VIEW . "</a>]";
						echo "</small></td>";
					}
				}
				else {

					if ($fm_cfg['show']['size']) {
						echo '<td></td>';
					}

					if ($fm_cfg['show']['date']) {
						echo '<td></td>';
					}

					if ($rename_column) {
						echo '<td><small>';
						if ($fm_cfg['perm']['dir']['rename'])
							echo "[<a href=\"$PHP_SELF_origvars_ue"
								. "fm_dir=$fm_dir_ue"
								. "&fm_action=confirm_rename_directory"
								. "&fm_filename=$entry_ue\">"
								. _FM_DIR_RENAME
								. "</a>]";
						echo '</small></td>';
					}

					if ($delete_column) {
						echo '<td><small>';
						if ($fm_cfg['perm']['dir']['remove'])
							echo "[<a href=\"$PHP_SELF_origvars_ue"
								. "fm_dir=$fm_dir_ue"
								. "&fm_action=confirm_remove_directory"
								. "&fm_filename=$entry_ue\">"
								. _FM_DIR_REMOVE
								."</a>]";
						echo '</small></td>';
					}

					if ($view_column) {
						echo '<td><small>';
						if ($fm_cfg['perm']['dir']['enter']
								&& ($fm_cfg['perm']['dir']['follow_symlinks']
									|| ! is_link($entry)))
							echo "[<a href=\"$PHP_SELF_origvars_ue" 
								. "fm_dir=$fm_dir_ue$entry_ue\">"
								. _FM_DIR_ENTER
								."</a>]";
						echo '</small></td>';
					}
				}

				echo "</tr>\n";
			}
		}

		$d->close();
	}

	if ($last_row) {
		echo '<tr><td colspan=7 align=center><small>';

		if ($fm_cfg['perm']['dir']['create'])
			echo "<br>[<a href=$PHP_SELF_origvars_ue"
				. "fm_dir=$fm_dir_ue"
				. "&fm_action=confirm_create_directory>"
				. _FM_DIR_CREATE
				. "</a>]"; 

		if ($fm_cfg['perm']['file']['create'])
			echo "&nbsp;[<a href=$PHP_SELF_origvars_ue"
				. "fm_dir=$fm_dir_ue"
				. "&fm_action=confirm_create_file>"
				. _FM_FILE_CREATE
				. "</a>]"; 

		if ($fm_cfg['perm']['file']['upload'])
			echo "&nbsp;[<a href=$PHP_SELF_origvars_ue"
				. "fm_dir=$fm_dir_ue"
				. "&fm_action=confirm_upload_file>"
				. _FM_FILE_UPLOAD
				. "</a>]"; 

		echo '</small></td></tr>';
	}

	echo '</table>';

}

echo '</form>';

/* Now return back to the initial working directory. */
chdir($fm_orig_dir);

if (defined('LOADED_AS_MODULE')) {
	CloseTable();
	include 'footer.php'; 
}

?>
